@@ -8,8 +8,8 @@
 ### algorithm (cf. Daley & Vere-Jones, 2003, Algorithm 7.5.V.).
 ###
 ### Copyright (C) 2010-2012 Sebastian Meyer
-### $Revision: 472 $
-### $Date: 2012-12-11 00:42:11 +0100 (Di, 11. Dez 2012) $
+### $Revision: 516 $
+### $Date: 2013-02-25 22:23:29 +0100 (Mo, 25. Feb 2013) $
 ################################################################################
 
 ### CAVE:
@@ -157,9 +157,9 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
 
     ### Check mark-generating function
 
-    # eps.t and eps.s are also taken to be unpredictable marks (generated by rmarks)
+    # eps.t and eps.s are also unpredictable marks (generated by rmarks)
     unpredMarks <- unique(c("eps.t", "eps.s", if (hase) {
-        setdiff(all.vars(epidemic), "type")
+        setdiff(all.vars(epidemic), c("type", names(stgrid)))
     }))
     rmarks <- match.fun(rmarks)
     sampleCoordinate <- coordinates(spsample(tiles, n=1L, type="random"))
@@ -168,13 +168,14 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     if (!is.data.frame(sampleMarks) || nrow(sampleMarks) != 1L) {
         stop("'rmarks' must return a one-row data.frame of marks")
     }
-    if (!all(sapply(sampleMarks, function(x) inherits(x, c("integer","numeric","factor"), which=FALSE)))) {
-        stop("'rmarks' must return \"numeric\", \"integer\" or \"factor\" variables only")
-    }
     markNames <- names(sampleMarks)
     if (.idx <- match(FALSE, unpredMarks %in% markNames, nomatch=0L)) {
         stop("the unpredictable mark '", unpredMarks[.idx], "' is not returned by 'rmarks'")
     }
+    if (!all(sapply(sampleMarks[unpredMarks], function(x)
+                    inherits(x, c("integer","numeric","logical","factor"), which=FALSE))))
+        warning("'rmarks' should return \"numeric\", \"logical\", or",
+                " \"factor\" ('epidemic') variables only")
 
 
     ### Check events (prehistory of the process)
@@ -192,8 +193,10 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     Nout <- 0L
     # do we have a prehistory in 'events'
     if (!missing(events) && !is.null(events)) {
-        .reservedColsIdx <- match(c("ID",".obsInfLength",".bdist",".influenceRegion",".sources"), names(events))
-        events <- events[setdiff(seq_along(names(events)),.reservedColsIdx)]
+        .reservedColsNames <- c("ID", ".obsInfLength", ".bdist",
+                                ".influenceRegion", ".sources",
+                                "BLOCK", "start") # from stgrid
+        events <- events[!names(events) %in% .reservedColsNames]
         if (!.skipChecks) {
             cat("Checking 'events':\n")
             events <- checkEvents(events, dropTypes = FALSE)
@@ -216,18 +219,19 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
             # update ID counter
             eventData$ID <- 1:Nout
             # check presence of unpredictable marks
-            if (.idx <- match(FALSE, unpredMarks %in% names(eventData), nomatch=0L)) {
-                stop("missing unpredictable mark '", unpredMarks[.idx], "' in 'events'")
+            if (length(.idx <- which(!unpredMarks %in% names(eventData)))) {
+                stop("missing unpredictable marks in 'events': ",
+                     paste0("\"", unpredMarks[.idx], "\"", collapse=", "))
             }
             # check type of unpredictable marks
             for (um in unpredMarks) {
-                if (!identical(class(sampleMarks[[um]]), class(eventData[[um]]))) {
-                    stop("the class of the unpredictable mark '", um, "' in the 'events' prehistory ",
+                if (!identical(class(sampleMarks[[um]]), class(eventData[[um]])))
+                    stop("the class of the unpredictable mark '", um,
+                         "' in the 'events' prehistory ",
                          "is not identical to the class returned by 'rmarks'")
-                }
             }
-            # add marks which are not in epidemic model but which are simulated by 'rmarks'
-            if (length(.add2events <- setdiff(markNames, names(eventData))) > 0L) {
+            # add marks which are not in the prehistory but simulated by 'rmarks'
+            if (length(.add2events <- setdiff(markNames, names(eventData)))) {
                 eventData <- cbind(eventData, sampleMarks[.add2events])
                 is.na(eventData[.add2events]) <- TRUE
             }
@@ -239,6 +243,19 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     }
 
 
+    ## helper function to attach covariates from 'stgrid' to events
+    attachstgridvars <- function (eventData, stgridvars)
+    {
+        gridcellsOfEvents <- integer(nrow(eventData))
+        for (i in seq_along(gridcellsOfEvents)) {
+            gridcellsOfEvents[i] <- gridcellOfEvent(eventData[i,"time"],
+                                                    eventData[i,"tile"],
+                                                    stgrid)
+        }
+        cbind(eventData, stgrid[gridcellsOfEvents, stgridvars, drop=FALSE])
+    }
+
+
     ### Build epidemic model matrix
 
     epidemic <- terms(epidemic, data = eventData, keep.order = TRUE)
@@ -249,14 +266,21 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     # helper function taking eventData and returning the epidemic model.matrix
     buildmme <- function (eventData)
     {
-        mfe <- model.frame(epidemic, data = eventData, na.action = na.fail, drop.unused.levels = FALSE)
+        # which variables do we have to copy from stgrid?
+        stgridCopyCols <- match(all.vars(epidemic), names(stgrid), nomatch = 0L)
+        eventData <- attachstgridvars(eventData, stgridCopyCols)
+        mfe <- model.frame(epidemic, data = eventData,
+                           na.action = na.fail, drop.unused.levels = FALSE)
         model.matrix(epidemic, mfe)
     }
     mme <- buildmme(eventData)
 
     if (ncol(mme) != q) {
-        cat(ncol(mme), "epidemic model terms:\t", paste(colnames(mme), collapse="  "), "\n")
-        stop("length of 'gamma' (", q, ") does not match the 'epidemic' specification (", ncol(mme), ")")
+        cat(ncol(mme), "epidemic model terms:\t",
+            paste(colnames(mme), collapse="  "), "\n")
+        stop("length of 'gamma' (", q,
+             ") does not match the 'epidemic' specification (",
+             ncol(mme), ")")
     }
 
 
@@ -291,20 +315,17 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     # ensure that we have correct contrasts in the endemic component
     attr(endemic, "intercept") <- as.integer(nbeta0 > 0L)
 
-    # which variables do we have to copy from stgrid?
-    stgridCopyCols <- match(all.vars(endemic), names(stgrid), nomatch = 0L)
-
-    # helper function taking eventData (with time and tile columns) and returning the endemic model.matrix
+    # helper function taking eventData (with time and tile columns)
+    # and returning the endemic model.matrix
     buildmmh <- function (eventData)
     {
         # if 'pi' appears in 'endemic' we don't care, and if a true covariate is
         # missing, model.frame will throw an error
-        # attach endemic covariates from 'stgrid' to events
-        gridcellsOfEvents <- integer(nrow(eventData))
-        for (i in seq_along(gridcellsOfEvents)) {
-            gridcellsOfEvents[i] <- gridcellOfEvent(eventData[i,"time"], eventData[i,"tile"], stgrid)
-        }
-        eventData <- cbind(eventData, stgrid[gridcellsOfEvents, stgridCopyCols, drop=FALSE])
+        # which variables do we have to copy from stgrid?
+        stgridCopyCols <- match(all.vars(endemic), names(stgrid), nomatch = 0L)
+
+        # attaching covariates from 'stgrid' to events
+        eventData <- attachstgridvars(eventData, stgridCopyCols)
         # construct model matrix
         mfhEvents <- model.frame(endemic, data = eventData, na.action = na.fail, drop.unused.levels = FALSE)
         mmhEvents <- model.matrix(endemic, mfhEvents)
@@ -345,14 +366,14 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     if (hase) {
 
         ## Check interaction functions
-        siaf <- do.call(".parseiaf", args = alist(siaf))
+        siaf <- do.call(".parseiaf", args = alist(siaf, "siaf", trace > 0))
         constantsiaf <- attr(siaf, "constant")
         if (siaf$npars != nsiafpars) {
             stop("length of 'siafpars' (", nsiafpars,
                  ") does not match the 'siaf' specification (", siaf$npars, ")")
         }
         
-        tiaf <- do.call(".parseiaf", args = alist(tiaf))
+        tiaf <- do.call(".parseiaf", args = alist(tiaf, "tiaf", trace > 0))
         constanttiaf <- attr(tiaf, "constant")
         if (constanttiaf) gmax <- 1L
         if (tiaf$npars != ntiafpars) {
@@ -897,7 +918,9 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
         if (length(ret) == 0L) NA_integer_ else ret
         #<- events of the prehistory have missing BLOCKs, thus return NA
     })
-    stgridIgnoreCols <- match(c("BLOCK",setdiff(obligColsNames_stgrid, "start")), names(stgrid))
+    stgridIgnoreCols <- match(c("BLOCK",
+                                setdiff(obligColsNames_stgrid, "start")),
+                              names(stgrid))
     eventData <- cbind(eventData, stgrid[stgrididx, -stgridIgnoreCols])
     rownames(eventData) <- eventData$ID
 
@@ -906,6 +929,7 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     eventData$.sources <- sources[seqAlongEvents]
     eventData$.bdist <- bdists[seqAlongEvents]
     eventData$.influenceRegion <- influenceRegions[seqAlongEvents]
+    attr(eventData$.influenceRegion, "nCircle2Poly") <- nCircle2Poly
 
 
     ### Construct "epidataCS" object
@@ -1015,7 +1039,8 @@ intensityplot.simEpidataCS <- function (x, ...)
 simulate.twinstim <- function (object, nsim = 1, seed = NULL, data, tiles,
     rmarks = NULL, t0 = NULL, T = NULL, nEvents = 1e5,
     control.siaf = object$control.siaf,
-    W = NULL, trace = FALSE, nCircle2Poly = 32, gmax = NULL, .allocate = 500, simplify = TRUE, ...)
+    W = NULL, trace = FALSE, nCircle2Poly = NULL, gmax = NULL,
+    .allocate = 500, simplify = TRUE, ...)
 {
     ptm <- proc.time()[[3]]
     cl <- match.call()
@@ -1043,6 +1068,8 @@ simulate.twinstim <- function (object, nsim = 1, seed = NULL, data, tiles,
     nsim <- as.integer(nsim)
     if (is.null(t0)) t0 <- object$timeRange[1]
     if (is.null(T))  T  <- object$timeRange[2]
+    if (is.null(nCircle2Poly))
+        nCircle2Poly <- attr(data$events$.influenceRegion, "nCircle2Poly")
 
 
     ### Retrieve arguments for simulation
